package com.fundi.framework.common.cslcomms;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;

import com.fundi.framework.common.nl1.Messages;

public class CSLResponse implements Serializable {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private static final long serialVersionUID = 1L;
	
	private static final Pattern	msgPattern = Pattern.compile("(</msg>)\\s*(<msg>)|<msg>|</msg>"); //$NON-NLS-1$

	private ArrayList<CSLFieldDefn> defns = new ArrayList<CSLFieldDefn>();
	
	//NOTE:
	//
	//The xml may be from more than one actual CSL response.  This can occur if a GPL function runs
	//multiple commands through the GPDriver class as one "logical" command.
	private String xml;
	private int currObjPos = -1;
	private boolean type2Response = true;
	private String cmd;
	private Date timeRun = null;
	
	private String returnCode = ""; //$NON-NLS-1$
	private String reasonCode = ""; //$NON-NLS-1$
	private String reasonMsg = ""; //$NON-NLS-1$
	
	private Xheader xhdr = null;
	
	//Route list does not come from the CSL response itself, but can be set by whoever ran the command.
	private String[] routeList = null;
	
	private boolean hasDataObjects = true;
	
	private Object provider = null;  //Would be a repository (for GPL commands) or an IMSplex (for DRD commands).
	
	private ArrayList<CSLObject> allObjects = null;
	
	public CSLResponse() {
		
		super();
	}
	
	public CSLResponse(String xml, String cmd) {
		
		super();
		
		int delimStart;
		int delimEnd;
		CSLFieldDefn defn;
		Iterator<CSLFieldDefn> itFlds;
		boolean found;
		
		this.xml = xml;
		this.cmd = cmd;
		
		delimStart = xml.indexOf(Messages.getString("CSLResponse_7")); //$NON-NLS-1$
		if (delimStart >= 0) {
			type2Response = true;
			delimStart = xml.indexOf(Messages.getString("CSLResponse_8")); //$NON-NLS-1$
			if (delimStart >= 0) {
				while (delimStart >= 0) {
					delimEnd = xml.indexOf(Messages.getString("CSLResponse_9"), delimStart) + 2; //$NON-NLS-1$
					defn = new CSLFieldDefn(xml.substring(delimStart, delimEnd));
					//Check the field defn. hasn't already been obtained (may already be there if running multiple commands).
					found = false;
					for (itFlds = defns.iterator(); itFlds.hasNext();) {
						if (itFlds.next().getSlbl().equals(defn.getSlbl())) {
							found = true;
							break;
						}
					}
					if (!found) {
						defns.add(defn);
					}
					delimStart = xml.indexOf(Messages.getString("CSLResponse_10"), delimEnd); //$NON-NLS-1$
				}
			}
		}
		else {
			delimStart = xml.indexOf(Messages.getString("CSLResponse_11")); //$NON-NLS-1$
			if (delimStart >= 0) {
				type2Response = false;
				//It's a Type-1 command response.
				defn = new CSLFieldDefn();
				defn.setSlbl(Messages.getString("CSLResponse_12")); //$NON-NLS-1$
				defn.setLlbl(Messages.getString("CSLResponse_13")); //$NON-NLS-1$
				defn.setAlign(CSLFieldDefn.ALIGN_LEFT);
				defn.setDtype(CSLFieldDefn.DTYPE_CHAR);
				defn.setKey(false);
				defn.setLen(8);
				defns.add(defn);
				defn = new CSLFieldDefn();
				defn.setSlbl(Messages.getString("CSLResponse_14")); //$NON-NLS-1$
				defn.setLlbl(Messages.getString("CSLResponse_15")); //$NON-NLS-1$
				defn.setAlign(CSLFieldDefn.ALIGN_LEFT);
				defn.setDtype(CSLFieldDefn.DTYPE_CHAR);
				defn.setKey(false);
				defn.setLen(80);
				defns.add(defn);
			}
		}
	}
	
	/**
	 * Currently this method is the target of a lot of calls during the processing of a request. Therefore it makes a big difference to performance to 
	 * cache the output result and only parse the data once.
	 * 
	 * @return an ArrayList of all the CSLObjects in the response.
	 */
	public synchronized ArrayList<CSLObject> getAllObjects() {
		if (allObjects == null) {
			ArrayList<CSLObject> objects = new ArrayList<CSLObject>();
			CSLObject obj;
			
			obj = nextObject();
			while (obj != null) {
				objects.add(obj);
				obj = nextObject();
			}
			
			allObjects = objects;
		}
		
		return allObjects;
	}
	
	public List<String> getMessageData() {
		String					msgData;
		String[]				msgs;
		List<String>			msgList = new ArrayList<String>();

		msgData = getFieldValue("msgdata"); //$NON-NLS-1$
		if (msgData != null) {
			msgs = msgPattern.split(msgData);
			msgList.addAll(Arrays.asList(msgs));
		}
		
		return msgList;
	}

	public CSLObject nextObject() {
		
		int endPos;
		int msgPos;
		CSLObject ret;
		
		if (xml ==  null || !hasDataObjects) {
			return null;
		}
		
		//If a Type-2 command response, the object definition is well defined.
		//
		//If a Type-1 command response, we use the <mbr> tag to define the object, but we get multiple fields of the same name,
		//one for each <msg> tag within the <mbr>.
		if (type2Response) {
			currObjPos = xml.indexOf(Messages.getString("CSLResponse_16"), currObjPos +1); //$NON-NLS-1$
			if (currObjPos >= 0) {
				currObjPos = currObjPos + 5;
				endPos = xml.indexOf(Messages.getString("CSLResponse_17"), currObjPos); //$NON-NLS-1$
				if (currObjPos < xml.length() && endPos > currObjPos) {
					ret = new CSLObject(defns, xml.substring(currObjPos, endPos).trim(), true, cmd);
					ret.setProvider(getProvider());
					return ret;
				}
				else {
					return null;  //This should only happen if we reach the IMS/CSL known problem with too many objects.
				}
			}
			else {
				return null;
			}
		}
		else {
			
			msgPos = xml.indexOf(Messages.getString("CSLResponse_18"), currObjPos +1); //$NON-NLS-1$
			currObjPos = xml.indexOf(Messages.getString("CSLResponse_19"), currObjPos +1); //$NON-NLS-1$
			if (currObjPos >= 0) {
				if (msgPos < currObjPos) {
					currObjPos = msgPos;
				}
			}
			else {
				if (msgPos >= 0) {
					currObjPos = msgPos;
				}
			}
			if (currObjPos >= 0) {
				endPos = xml.indexOf(Messages.getString("CSLResponse_20"), currObjPos); //$NON-NLS-1$
				ret = new CSLObject(defns, xml.substring(currObjPos, endPos).trim(), false, cmd);
				ret.setProvider(getProvider());
				currObjPos = endPos + 5;
				return ret;
			}
			else {
				return null;
			}
		}
	}
	
	public String getFieldValue(String fldName) {

		//This method will provide any XML field as a string value.
		//
		//It's main use is for accessing control fields.  Beware of fields which have multiple occurrences, as only the first value will be returned.
		
		int pos;
		int valueStart;
		
		pos = xml.indexOf(Messages.getString("CSLResponse_21") + fldName + Messages.getString("CSLResponse_22")); //$NON-NLS-1$ //$NON-NLS-2$
		if (pos >= 0) {
			valueStart = pos + fldName.length() + 2;
			return xml.substring(valueStart, xml.indexOf(Messages.getString("CSLResponse_23") + fldName + Messages.getString("CSLResponse_24"), valueStart)); //$NON-NLS-1$ //$NON-NLS-2$
		}
		else {
			return ""; //$NON-NLS-1$
		}
	}
	
	public ArrayList<CSLFieldDefn> getFieldDefns() {
		
		return defns;
	}

	public boolean isType2Response() {
		
		return type2Response;
	}

	public Date getTimeRun() {
		return timeRun;
	}

	public void setTimeRun(Date timeRun) {
		this.timeRun = timeRun;
	}

	public String getReturnCode() {
		return returnCode;
	}

	public void setReturnCode(String returnCode) {
		this.returnCode = returnCode;
	}

	public String getReasonCode() {
		return reasonCode;
	}

	public void setReasonCode(String reasonCode) {
		this.reasonCode = reasonCode;
	}

	public String getReasonMsg() {
		return reasonMsg;
	}

	public void setReasonMsg(String reasonMsg) {
		this.reasonMsg = reasonMsg;
	}

	public String getCmd() {
		return cmd;
	}

	public String[] getRouteList() {
		
		return routeList;
	}

	public void setRouteList(String[] routeList) {
		
		this.routeList = routeList;
	}
	
	public String getRouteListAsString() {
		
		String list = ""; //$NON-NLS-1$
		int cnt;
		
		for (cnt = 0; cnt < routeList.length; cnt++) {
			if (list.length() > 0) {
				list = list + Messages.getString("CSLResponse_27"); //$NON-NLS-1$
			}
			list = list + routeList[cnt];
		}
		
		return list;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((cmd == null) ? 0 : cmd.hashCode());
		result = prime * result + currObjPos;
		result = prime * result + ((defns == null) ? 0 : defns.hashCode());
		result = prime * result
				+ ((reasonCode == null) ? 0 : reasonCode.hashCode());
		result = prime * result
				+ ((reasonMsg == null) ? 0 : reasonMsg.hashCode());
		result = prime * result
				+ ((returnCode == null) ? 0 : returnCode.hashCode());
		result = prime * result + Arrays.hashCode(routeList);
		result = prime * result + ((timeRun == null) ? 0 : timeRun.hashCode());
		result = prime * result + (type2Response ? 1231 : 1237);
		result = prime * result + ((xml == null) ? 0 : xml.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CSLResponse other = (CSLResponse) obj;
		if (cmd == null) {
			if (other.cmd != null)
				return false;
		} else if (!cmd.equals(other.cmd))
			return false;
		if (currObjPos != other.currObjPos)
			return false;
		if (defns == null) {
			if (other.defns != null)
				return false;
		} else if (!defns.equals(other.defns))
			return false;
		if (reasonCode == null) {
			if (other.reasonCode != null)
				return false;
		} else if (!reasonCode.equals(other.reasonCode))
			return false;
		if (reasonMsg == null) {
			if (other.reasonMsg != null)
				return false;
		} else if (!reasonMsg.equals(other.reasonMsg))
			return false;
		if (returnCode == null) {
			if (other.returnCode != null)
				return false;
		} else if (!returnCode.equals(other.returnCode))
			return false;
		if (!Arrays.equals(routeList, other.routeList))
			return false;
		if (timeRun == null) {
			if (other.timeRun != null)
				return false;
		} else if (!timeRun.equals(other.timeRun))
			return false;
		if (type2Response != other.type2Response)
			return false;
		if (xml == null) {
			if (other.xml != null)
				return false;
		} else if (!xml.equals(other.xml))
			return false;
		return true;
	}

	public boolean hasDataObjects() {

		return hasDataObjects;
	}

	public void setHasDataObjects(boolean hasDataObjects) {
		
		this.hasDataObjects = hasDataObjects;
	}

	public Xheader getXhdr() {
		return xhdr;
	}

	public void setXhdr(Xheader xhdr) {
		this.xhdr = xhdr;
	}

	public Object getProvider() {
		
		return provider;
	}

	public void setProvider(Object responseSource) {
		
		this.provider = responseSource;
	}

	public String getXml() {
		return xml;
	}

}
